

LIBRARY IEEE;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_1164.all;


ENTITY PIout_to_PWM_Value IS

GENERIC(

	PWM_min				: integer	:= 0;			-- Min. PWM value in %
	PWM_max				: integer	:= 65			-- Max. PWM value in %
);

PORT(
	i_System_Clock	: IN	std_logic;
	i_n_Reset		: IN	std_logic;								-- Reset
	i_PWM_Freq_Devider: IN  integer range 0 to 65535;      -- PWM frequency devider
	i_PI_out			: IN	std_logic_vector(15 downto 0);	
	i_UpdatePWM		: IN	std_logic;								-- Update the PWM value
	o_PWMvalue		: OUT	integer range 0 to 8192				-- PWM value in 100%/PWM_Freq_Devider
);

END ENTITY PIout_to_PWM_Value;

ARCHITECTURE behaviour OF PIout_to_PWM_Value IS
	constant c_PI_Resolution	: integer	:= 16;	
	signal 	s_PWM_int			: std_logic_vector(c_PI_Resolution+15 downto 0);
	
BEGIN

ASSERT (PWM_min >= 0 AND PWM_min < PWM_max)
	REPORT "PWM_min value is wrong! PWM_min = 0..PWM_max"
	SEVERITY error;
ASSERT (PWM_max > PWM_min AND PWM_max <= 100)
	REPORT "PWM_max value is wrong! PWM_min = PWM_min..100"
	SEVERITY error;

	
createPWMValue : PROCESS(i_n_Reset, i_System_Clock)
	variable v_CalcState	: unsigned (1 downto 0);
	variable v_PWMvalue	: std_logic_vector(c_PI_Resolution+6 downto 0);
	
	
BEGIN
	IF i_n_Reset = '0' THEN
		-- Asynchronous reset
		s_PWM_int	<= (others => '0');
		v_PWMvalue	:= (others => '0');
		v_CalcState	:= (others => '0');
	--	v_PWM_Freq_Devider := (others => '0');
	ELSIF rising_edge(i_System_Clock) THEN
		IF i_UpdatePWM = '1' OR v_CalcState /= "00" THEN
			v_CalcState	:= v_CalcState + 1;
		END IF;
		IF v_CalcState = "01" THEN
			IF i_PI_out(c_PI_Resolution-1) = '1' THEN
				s_PWM_int	<= (others => '0');
				v_PWMvalue	:= (others => '0');
				v_CalcState	:= "00";
			ELSE
				v_PWMvalue	:= std_logic_vector(unsigned(i_PI_out) * (to_unsigned(PWM_max, 7) - to_unsigned(PWM_min, 7)));
			END IF;
		ELSIF v_CalcState = "10" THEN
			v_PWMvalue	:= std_logic_vector(unsigned(v_PWMvalue) + to_unsigned(PWM_min, 7));
		ELSIF v_CalcState = "11" THEN
			s_PWM_int	<= std_logic_vector(unsigned(v_PWMvalue) * to_unsigned(i_PWM_Freq_Devider / 100, 9)); -- Timing Fails
			v_CalcState	:= "00";
		END IF;
	END IF;
END PROCESS createPWMValue;

setPWMValue : PROCESS(i_n_Reset, i_System_Clock)
BEGIN
	IF i_n_Reset = '0' THEN
		-- Asynchronous reset
		o_PWMvalue	<= 0;
	ELSIF rising_edge(i_System_Clock) THEN
		o_PWMvalue	<= to_integer(unsigned(s_PWM_int(c_PI_Resolution+11 downto c_PI_Resolution-1)));
	END IF;
END PROCESS setPWMValue;

END ARCHITECTURE behaviour;